#include "stm32f10x.h"
#include "config.h"
#include "util.h"
#include <string.h>
#include <stdio.h>
	
void SPI2_Init(void)
{
	SPI_InitTypeDef   SPI_InitStructure;

	  /* SPI Config -------------------------------------------------------------*/
	  SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
	  SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
	  SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
	  SPI_InitStructure.SPI_CPOL = SPI_CPOL_Low;
	  SPI_InitStructure.SPI_CPHA = SPI_CPHA_1Edge;
	  SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
	  SPI_InitStructure.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_128;
	  SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
	  SPI_InitStructure.SPI_CRCPolynomial = 7;

	  SPI_Init(SPI2, &SPI_InitStructure);
	  
	  /* Enable SPI */
	  SPI_Cmd(SPI2, ENABLE);

}


uint8_t SPI2_SendByte(uint8_t byte)
{
	  while (SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_TXE) == RESET);

	  SPI_I2S_SendData(SPI2, byte);
	  while (SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_RXNE) == RESET);

	  return SPI_I2S_ReceiveData(SPI2);

}



u8 SpiByteStuff(u8 *pNextData)
 {
 if( (SPI_ESC_CHAR	== *pNextData) ||
	 (SPI_XON_CHAR	== *pNextData) ||
	 (SPI_XOFF_CHAR == *pNextData) ||
	 (SPI_IDLE_CHAR == *pNextData) ||
	 (SPI_INVALID_CHAR_ALL_ONE == *pNextData) ||
	 (SPI_INVALID_CHAR_ALL_ZERO == *pNextData)||
	 (SPI_LINK_READY == *pNextData))
 	{   // Byte stuffing is required
		 *pNextData = *pNextData ^ 0x20;
		 return 0x01;
	 }
	 else
	 {
		 return 0x00;
	 }

 } 



void WizFI210_Write_Buf(u8 *s)
{
 
	while (*s != '\0')
	{


		if (SpiByteStuff(s)) {
			// Byte stuffing is required.
			GPIO_ResetBits(GPIOB, SPI2_nSS); 
			SPI2_SendByte(SPI_ESC_CHAR);
			GPIO_SetBits(GPIOB, SPI2_nSS); 
		}

			GPIO_ResetBits(GPIOB, SPI2_nSS); 
			SPI2_SendByte(*s);
			GPIO_SetBits(GPIOB, SPI2_nSS); 
			s ++;
	}
  	
}

void WizFI210_Write_Byte(u8 data1)
{


	if (SpiByteStuff(&data1)) {
		GPIO_ResetBits(GPIOB, SPI2_nSS); 

		// Byte stuffing is required.
		SPI2_SendByte(SPI_ESC_CHAR);
		
		GPIO_SetBits(GPIOB, SPI2_nSS); 
		
	}


	GPIO_ResetBits(GPIOB, SPI2_nSS); 

	SPI2_SendByte(data1);
		
	GPIO_SetBits(GPIOB, SPI2_nSS); 

}

u8 WizFI210_Read_Byte(void)
{
	u8 data1;

	GPIO_ResetBits(GPIOB, SPI2_nSS); 

	data1 = SPI2_SendByte(0x00);

	GPIO_SetBits(GPIOB, SPI2_nSS); 

	return(data1);
}

u16 WizFI210_Read_Buf(u8 * buf)
{
	u16 idx = 0;
	u8 tmp;
	u8 chk = 0;
	
	while(GPIO_ReadInputDataBit(GPIOB, PB10) == 1) {

		tmp = WizFI210_Read_Byte();

		if ((tmp != SPI_IDLE_CHAR) &&  (tmp != SPI_INVALID_CHAR_ALL_ZERO) && (tmp != SPI_INVALID_CHAR_ALL_ONE))	{
			if (tmp == SPI_ESC_CHAR) {
				chk = 1;
				
			}else {
				if (chk == 1) {
					chk = 0;
					tmp = tmp ^ 0x20;
				}
				buf[idx++] = tmp;
			}
		}
	}

	return(idx);
}

u8 Check_OK(char * buf, u16 TimeOut)
{
	u16 idx = 0;
	u16 t=0;
	u8 tmp;
	char * ret = NULL;

	while(1) {

		tmp = WizFI210_Read_Byte();
		if ((tmp != SPI_IDLE_CHAR) &&  (tmp != SPI_INVALID_CHAR_ALL_ZERO) && (tmp != SPI_INVALID_CHAR_ALL_ONE)){
			buf[idx++] = tmp;
			
#ifdef DEBUG_MSG
			putchar(tmp);
#endif
			if (tmp == ']') break;
			
		}
		
		if (t++ > TimeOut) break;
		wait_1ms(1);
		
	}

	while(1) {
		tmp = WizFI210_Read_Byte();
		if ((tmp != SPI_IDLE_CHAR) &&  (tmp != SPI_INVALID_CHAR_ALL_ZERO) && (tmp != SPI_INVALID_CHAR_ALL_ONE)){
			buf[idx++] = tmp;
			
#ifdef DEBUG_MSG
			putchar(tmp);
#endif
		}
		if (GPIO_ReadInputDataBit(GPIOB, PB10) == 0) break;
		if (t++ > TimeOut) break;
		wait_1ms(1);
		
	}

	//check received message
	ret = strstr(buf, "[OK]");

#ifdef DEBUG_MSG
	printf("\r\n%s", buf);
#endif

	if (ret != NULL) return 1;
	else return 0;
			
}



