#include "stm32f10x.h"
#include "stdio.h"
#include "config.h"
#include "util.h"
#include "spi2.h"

#include <string.h>
#include <stdlib.h>


u8 INT_Enabled;
char Msgbuf[255];
char Recvbuf[BUF_SIZE];
u16 Recvlen;


#define SSID 			"WIZ_RED"
#define WPA_PASS 	"wiznet0123456"

#define Infrastructure	0
#define Ad_hoc		1

#define Client_Mode	0
#define Server_Mode	1

#define Protocol_UDP	0
#define Protocol_TCP	1


u8 IP[4] 		= {192, 168, 88, 123};
u8 Sub[4] 	= {255, 255, 255, 0};
u8 Gw[4] 		= {192, 168, 88, 1};
u8 SIP[4] 	= {192, 168, 88, 228};
u16 ListenPort = 5000;
u16 ServerPort = 5000;

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/


/* Private function prototypes -----------------------------------------------*/
void RCC_Configuration(void);
void NVIC_Configuration(void);
void GPIO_Configuration(void);


/*******************************************************************************
* Function Name  : main
* Description    : Main program.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
	u8 key;
	u16 cnt;
	
	INT_Enabled = 0;

	// Configure the system clocks
	RCC_Configuration();

	// GPIO Configuration
	GPIO_Configuration();

	 // NVIC Configuration
	NVIC_Configuration();

	// 115200
	USART1_Init();
	
	SPI2_Init();
     
        printf("\r\n=============== WizFi210 SPI Test ===================");
		
#ifdef USE_HW_RESET
        printf("\r\nReset WizFi210");
	// H/W reset the WizFi210
	GPIO_ResetBits(GPIOB, PB11);
	wait_10ms(1);
	GPIO_SetBits(GPIOB, PB11);
	wait_10ms(1);

	while(GPIO_ReadInputDataBit(GPIOB, PB10) == 0);

#else
	// check data mode(Initial reset) or not
	sprintf(Msgbuf, "AT\r\n");
	printf("\r\n[0] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 1000) == 0)  {
		// data mode
		sprintf(Msgbuf, "+++");
		WizFI210_Write_Buf(Msgbuf);
		wait_10ms(100);
	}

#endif


        printf("\r\nSend sync");
	WizFI210_Write_Byte(SPI_IDLE_CHAR); 

	
	// Flush the data
	printf("\r\nRead data");
	while(GPIO_ReadInputDataBit(GPIOB, PB10) == 1) {
		key = WizFI210_Read_Byte();
		if ((key != SPI_IDLE_CHAR) &&  (key != SPI_INVALID_CHAR_ALL_ZERO) && (key != SPI_INVALID_CHAR_ALL_ONE)) {
			putchar(key);
		}
	}

	// WiFi Configuration
	//
	printf("\r\nWifi Configuration");

	sprintf(Msgbuf, "AT\r\n");
	printf("\r\n[1] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 1000))  printf(" : Success");
	else {
		sprintf(Msgbuf, "AT\r\n");
		printf("\r\n[1] = %s", Msgbuf);
		WizFI210_Write_Buf(Msgbuf);
		memset(Msgbuf, '\0', sizeof(Msgbuf));
		if (Check_OK(Msgbuf, 1000))  printf(" : Success");
		else printf(" : Error");
		
	}

	
	//  AT+WD
	sprintf(Msgbuf, "AT+WD\r\n");
	printf("\r\n[2] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 1000)) printf(" : Success");
	else printf(" : Error");

	//  AT+WAUTO
	sprintf(Msgbuf, "AT+WAUTO=%d,%s\r\n", Infrastructure, SSID);
	printf("\r\n[3] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 1000)) printf(" : Success");
	else printf(" : Error");

	//  AT+WAUTH
	sprintf(Msgbuf, "AT+WAUTH=%d\r\n",0);
	printf("\r\n[4] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 1000)) printf(" : Success");
	else printf(" : Error");

	//  AT+WWPA
	sprintf(Msgbuf, "AT+WWPA=%s\r\n",WPA_PASS);
	printf("\r\n[5] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 1000)) printf(" : Success");
	else printf(" : Error");

	//  AT+NDHCP
	sprintf(Msgbuf, "AT+NDHCP=%d\r\n",0);
	printf("\r\n[6] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 1000)) printf(" : Success");
	else printf(" : Error");

	//  AT+NSET
	sprintf(Msgbuf, "AT+NSET=%d.%d.%d.%d,%d.%d.%d.%d,%d.%d.%d.%d\r\n", 
					IP[0],IP[1],IP[2],IP[3],Sub[0],Sub[1],Sub[2],Sub[3],Gw[0],Gw[1],Gw[2],Gw[3]);
	printf("\r\n[7] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 1000)) printf(" : Success");
	else printf(" : Error");

	//  AT+NAUTO
	sprintf(Msgbuf, "AT+NAUTO=%d,%d,,%d\r\n",Server_Mode, Protocol_TCP, ListenPort);
	printf("\r\n[8] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 1000)) printf(" : Success");
	else printf(" : Error");

	//  AT+XDUM
	sprintf(Msgbuf, "AT+XDUM=%d\r\n", 0); //  1: Uart message is disable, 0: enable
	printf("\r\n[9] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 1000)) printf(" : Success");
	else printf(" : Error");
	
	//  ATA
	sprintf(Msgbuf, "ATA\r\n");
	printf("\r\n[10] = %s", Msgbuf);
	WizFI210_Write_Buf(Msgbuf);
	memset(Msgbuf, '\0', sizeof(Msgbuf));
	if (Check_OK(Msgbuf, 20000)) printf(" : Success");
	else printf(" : Error");

	printf("\r\nConfiguration completed");
	
	INT_Enabled = 1;
	while(1) {

	 	// waitting for input data
		 cnt = 0;
		 memset(Msgbuf, '\0', sizeof(Msgbuf));
		 cnt = GetInputString(Msgbuf); // blocking function

		 printf("\r\nSend data [%d] Msg = %s", cnt, Msgbuf);
		 if (cnt > 0) WizFI210_Write_Buf(Msgbuf);
		 wait_10ms(1);


	}
}

void GPIO_Configuration(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	EXTI_InitTypeDef EXTI_InitStructure;	

  // Configure the GPIO ports( USART1 Transmit and Receive Lines)
  //  Configure the USART1_Tx as Alternate function Push-Pull 
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStructure.GPIO_Pin =  USART1_TX;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);
  
  // Configure the USART1_Rx as input floating
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_InitStructure.GPIO_Pin = USART1_RX;
  GPIO_Init(GPIOA, &GPIO_InitStructure);


 // SPI 2
  /* Configure SPIy pins: SCK, MISO and MOSI */
  GPIO_InitStructure.GPIO_Pin = SPI2_SCK | SPI2_MISO | SPI2_MOSI;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin = SPI2_nSS; 
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, SPI2_nSS);

	// Port B output
    GPIO_InitStructure.GPIO_Pin = PB11;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_SetBits(GPIOB, PB11);


	// Port B input
    GPIO_InitStructure.GPIO_Pin = PB10;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

#ifdef USE_INT
    GPIO_EXTILineConfig(GPIO_PortSourceGPIOB, GPIO_PinSource10);

    EXTI_InitStructure.EXTI_Line = EXTI_Line10;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;

    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;  
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);
#endif	

}


/*******************************************************************************
* Function Name  : RCC_Configuration
* Description    : Configures the different system clocks.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void RCC_Configuration(void)
{
ErrorStatus HSEStartUpStatus;
  
 /* RCC system reset(for debug purpose) */
  RCC_DeInit();

  /* Enable HSE */
  RCC_HSEConfig(RCC_HSE_ON);

  /* Wait till HSE is ready */
  HSEStartUpStatus = RCC_WaitForHSEStartUp();

  if(HSEStartUpStatus == SUCCESS)
  {
    /* Enable Prefetch Buffer */
    FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

    /* Flash 2 wait state */
    FLASH_SetLatency(FLASH_Latency_2);
 
    /* HCLK = SYSCLK */
    RCC_HCLKConfig(RCC_SYSCLK_Div1); 
  
    /* PCLK2 = HCLK */
    RCC_PCLK2Config(RCC_HCLK_Div1); 

    /* PCLK1 = HCLK/2 */
    RCC_PCLK1Config(RCC_HCLK_Div2);

    /* PLLCLK = 8MHz * 9 = 72 MHz */
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_9);

    /* Enable PLL */ 
    RCC_PLLCmd(ENABLE);

    /* Wait till PLL is ready */
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET)
    {
    }

    /* Select PLL as system clock source */
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

    /* Wait till PLL is used as system clock source */
    while(RCC_GetSYSCLKSource() != 0x08)
    {
    }
  }
   
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2 | RCC_APB1Periph_SPI2, ENABLE);

  RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO  | RCC_APB2Periph_USART1, ENABLE);



}

/*******************************************************************************
* Function Name  : NVIC_Configuration
* Description    : Configures Vector Table base location.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void NVIC_Configuration(void)
{

#ifdef USE_INT
  NVIC_InitTypeDef NVIC_InitStructure;
#endif

#ifdef  VECT_TAB_RAM  
  /* Set the Vector Table base location at 0x20000000 */ 
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);

#else  /* VECT_TAB_FLASH  */
  /* Configure one bit for preemption priority */
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

#ifdef USE_INT
   /* Enable the EXTI15_10 Interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
#endif

  /* Set the Vector Table base location at 0x08000000 */ 
NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
  
#endif
}



